#!/usr/bin/perl

# Exit codes:
# 0 = success
# 1 = failed to read config file
# 2 = failed to write config file
# 3 = no changes needed
# 4 = failed to restart daemon

use strict;
use warnings;

my $FILE='/etc/cups/cupsd.conf';

unless (open FILE,$FILE) {
  print "Failed to open the CUPS configuration file.\n";
  exit 2;
}
my @LINES=<FILE>;
close FILE;

my($exists,$commented,$insert);

for(my $i=0;$i<@LINES;$i++) {
  if($LINES[$i]=~/^\s*BrowseProtocols/i) {
    $exists=$i; # BrowseProtocols is already defined; replace first instance
    last;
  }
  if($LINES[$i]=~/BrowseProtocols/i) {
    $commented=$i; # BrowseProtocols appears to be commented out; insert immediately after last instance
  }
  if($i>0 && $LINES[$i-1]=~/^\s*BrowseAllow/ && $LINES[$i]=~/^\s*$/) {
    $insert=$i-1; # Insert after BrowseAllow line, if there's a blank line
  }
}

if($exists) {
  if($LINES[$exists]=~/^\s*BrowseProtocols\s+all\s*$/) {
    print "Your configuration appears to have been fixed already.  If it's not working, you may need to poke around in $FILE yourself, or you may have a completely different problem.\n";
    exit 1;
  }
  $LINES[$exists]='### '.$LINES[$exists];
  insert($exists);
} elsif($commented) {
  insert($commented);
} elsif($insert) {
  insert($insert);
} else {
  insert($#LINES); # append to end of file
}

if(system('/sbin/service','org.cups.cupsd','stop')) {
  print "The CUPS service could not be stopped.  You may need to reboot your Mac.\n";
  exit 4;
}
if(system('/sbin/service','org.cups.cupsd','start')) {
  print "The CUPS service could not be restarted.  You may need to reboot your Mac.\n";
  exit 4;
}
print "CUPS browsing has been enabled successfully.\n";
exit 0;

sub insert {
  my($after)=@_;
  unless(open FILE,'>',$FILE) {
    print "Failed to write to the CUPS configuration file.\n";
    exit 3;
  }
  for(my $i=0;$i<$after+1;$i++) {
    print FILE $LINES[$i];
  }
  
  # insert this:
  print FILE "\n";
  print FILE "### CUPS Browsing is disabled by default in Mac OS X 10.5;\n";
  print FILE "### this line fixes it:\n";
  print FILE "BrowseProtocols all\n";
  
  for(my $i=$after+1;$i<@LINES;$i++) {
    print FILE $LINES[$i];
  }
  close FILE;
}
